@@ -28,10 +28,12 @@
 
 #include <libxfce4panel/xfce-panel-plugin.h>
 
+#include "xkb-keyboard.h"
 #include "xkb-xfconf.h"
 
 void    xkb_dialog_configure_plugin    (XfcePanelPlugin *plugin,
-                                        XkbXfconf       *config);
+                                        XkbXfconf       *config,
+                                        XkbKeyboard     *keyboard);
 void    xkb_dialog_about_show          (void);
 
 #endif
