@@ -27,6 +27,11 @@
 #include <config.h>
 #endif
 
+#include <gtk/gtk.h>
+#ifdef GDK_WINDOWING_X11
+#include <gdk/gdkx.h>
+#endif
+
 #include <libxfce4ui/libxfce4ui.h>
 #include <librsvg/rsvg.h>
 #include <garcon/garcon.h>
@@ -111,8 +116,6 @@ static void         xkb_plugin_notify                   (XkbPlugin        *plugi
 
 static void         xkb_plugin_refresh_gui              (XkbPlugin        *plugin);
 
-static void         xkb_plugin_configure_layout         (GtkWidget        *widget);
-
 static gboolean     xkb_plugin_button_clicked           (GtkWidget        *widget,
                                                          GdkEventButton   *event,
                                                          XkbPlugin        *plugin);
@@ -183,6 +186,28 @@ xkb_plugin_construct (XfcePanelPlugin *plugin)
   GtkWidget      *configure_layouts;
   GtkCssProvider *css_provider;
 
+  xfce_textdomain (GETTEXT_PACKAGE, LOCALEDIR, "UTF-8");
+
+  /* abort on non X11 environments */
+#ifdef GDK_WINDOWING_X11
+  gboolean x11_windowing = GDK_IS_X11_DISPLAY (gdk_display_get_default ());
+#else
+  gboolean x11_windowing = FALSE;
+#endif
+  if (!x11_windowing)
+    {
+      GtkWidget *dialog = xfce_message_dialog_new (NULL, xfce_panel_plugin_get_display_name (plugin), "dialog-error",
+                                                   _("Unsupported windowing environment"), NULL,
+                                                   _("_OK"), GTK_RESPONSE_OK, NULL);
+      XFCE_PANEL_PLUGIN_GET_CLASS (plugin)->free_data = NULL;
+      XFCE_PANEL_PLUGIN_GET_CLASS (plugin)->orientation_changed = NULL;
+      XFCE_PANEL_PLUGIN_GET_CLASS (plugin)->size_changed = NULL;
+      gtk_dialog_run (GTK_DIALOG (dialog));
+      gtk_widget_destroy (dialog);
+      xfce_panel_plugin_remove (plugin);
+      return;
+    }
+
   xkb_plugin = XKB_PLUGIN (plugin);
 
   xkb_plugin->config = xkb_xfconf_new (xfce_panel_plugin_get_property_base (plugin));
@@ -244,8 +269,6 @@ xkb_plugin_construct (XfcePanelPlugin *plugin)
   g_signal_connect_swapped (G_OBJECT (xkb_plugin->modifier), "modifier-changed",
                             G_CALLBACK (xkb_plugin_modifier_changed), xkb_plugin);
 
-  xfce_textdomain (GETTEXT_PACKAGE, LOCALEDIR, "UTF-8");
-
   xfce_panel_plugin_menu_show_configure (plugin);
   xfce_panel_plugin_menu_show_about (plugin);
 
@@ -322,7 +345,8 @@ xkb_plugin_configure_plugin (XfcePanelPlugin *plugin)
 {
   XkbPlugin *xkb_plugin = XKB_PLUGIN (plugin);
 
-  xkb_dialog_configure_plugin (plugin, xkb_plugin->config);
+  xkb_dialog_configure_plugin (plugin,
+                               xkb_plugin->config, xkb_plugin->keyboard);
 }
 
 
@@ -481,13 +505,15 @@ xkb_plugin_popup_menu_show (GtkWidget      *widget,
                             XkbPlugin      *plugin)
 {
   gtk_widget_set_state_flags (widget, GTK_STATE_FLAG_CHECKED, FALSE);
-#if GTK_CHECK_VERSION(3, 22, 0)
-  gtk_menu_popup_at_widget (GTK_MENU (plugin->popup), widget,
-                            GDK_GRAVITY_NORTH_WEST, GDK_GRAVITY_NORTH_WEST, (GdkEvent *) event);
+#if LIBXFCE4PANEL_CHECK_VERSION (4, 17 ,2)
+  xfce_panel_plugin_popup_menu (XFCE_PANEL_PLUGIN (plugin), GTK_MENU (plugin->popup),
+                                widget, (GdkEvent *) event);
 #else
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   gtk_menu_popup (GTK_MENU (plugin->popup), NULL, NULL,
                   xfce_panel_plugin_position_menu, xkb_plugin_get_plugin (plugin),
                   0, event->time);
+G_GNUC_END_IGNORE_DEPRECATIONS
 #endif
 }
 
@@ -550,7 +576,7 @@ xkb_plugin_refresh_gui (XkbPlugin *plugin)
 
 
 
-static void
+void
 xkb_plugin_configure_layout (GtkWidget *widget)
 {
   gchar           *desktop_file;
@@ -567,13 +593,14 @@ xkb_plugin_configure_layout (GtkWidget *widget)
   if (item)
     {
       g_shell_parse_argv (garcon_menu_item_get_command (item), NULL, &argv, &error);
-      succeed = xfce_spawn_on_screen (gtk_widget_get_screen (widget),
-                                      garcon_menu_item_get_path (item),
-                                      argv, NULL, G_SPAWN_SEARCH_PATH,
-                                      garcon_menu_item_supports_startup_notification (item),
-                                      gtk_get_current_event_time (),
-                                      garcon_menu_item_get_icon_name (item),
-                                      &error);
+      succeed = xfce_spawn (gtk_widget_get_screen (widget),
+                            garcon_menu_item_get_path (item),
+                            argv, NULL, G_SPAWN_SEARCH_PATH,
+                            garcon_menu_item_supports_startup_notification (item),
+                            gtk_get_current_event_time (),
+                            garcon_menu_item_get_icon_name (item),
+                            TRUE,
+                            &error);
       g_strfreev (argv);
       garcon_menu_item_unref (item);
       g_assert (succeed);
@@ -685,6 +712,7 @@ xkb_plugin_layout_image_draw (GtkWidget *widget,
   gint                  display_scale;
   gboolean              caps_lock_indicator;
   gboolean              caps_lock_enabled;
+  PangoContext         *pc;
 
   display_type = xkb_xfconf_get_display_type (plugin->config);
   display_name = xkb_xfconf_get_display_name (plugin->config);
@@ -729,12 +757,13 @@ xkb_plugin_layout_image_draw (GtkWidget *widget,
 
     case DISPLAY_TYPE_SYSTEM:
       gtk_style_context_get (style_ctx, state, "font", &desc, NULL);
+      pc = gtk_widget_get_pango_context (widget);
 
       xkb_cairo_draw_label_system (cr, group_name,
                                    actual_hsize, actual_vsize,
                                    variant_index,
                                    caps_lock_indicator && caps_lock_enabled,
-                                   desc, rgba);
+                                   desc, pc, rgba);
       break;
     }
 
